from modeltranslation.translator import register, TranslationOptions
from lite_cms_core.models import BaseEntity, ContentFieldMixin


@register(BaseEntity)
class TranslatedBaseEntity(TranslationOptions):
    fields = ('title', )


@register(ContentFieldMixin)
class TranslatedContentFieldMixin(TranslationOptions):
    fields = ('content', )



from django import template
from django.apps import apps
from django.contrib.contenttypes.models import ContentType
from django.urls import reverse
from django.conf import settings

register = template.Library()


@register.inclusion_tag('lite_cms_core/includes/edit_toolbar.html')
def edit_toolbar(model_instance=None, permission=False):
    """
    Shows the editable toolbar and adds an admin edit link
    if model instance is not None
    """
    admin_edit_link = None
    if model_instance and permission:
        edit_link_name = f'admin:{model_instance._meta.app_label}_{model_instance._meta.model_name}_change'
        admin_edit_link = reverse(edit_link_name, args=[model_instance.pk])
    return {'admin_edit_link': admin_edit_link}


@register.inclusion_tag("lite_cms_core/includes/search_form.html", takes_context=True)
def search_form(context, search_model_names=None):
    """
    Includes the search form with a list of models to use as choices
    for filtering the search by. Models should be a string with models
    in the format ``app_label.model_name`` separated by spaces. The
    string ``all`` can also be used, in which case the models defined
    by the ``SEARCH_MODEL_CHOICES`` setting will be used.
    """
    template_vars = {
        "request": context["request"],
    }
    if not search_model_names or not settings.SEARCH_MODEL_CHOICES:
        search_model_names = []
    elif search_model_names == "all":
        search_model_names = list(settings.SEARCH_MODEL_CHOICES)
    else:
        search_model_names = search_model_names.split(" ")
    search_model_choices = []
    for model_name in search_model_names:
        try:
            model = apps.get_model(*model_name.split(".", 1))
        except LookupError:
            pass
        else:
            verbose_name = model._meta.verbose_name_plural.capitalize()
            search_model_choices.append((verbose_name, model_name))
    template_vars["search_model_choices"] = sorted(search_model_choices)
    return template_vars


@register.filter
def get_search_type(instance):
    typus = ContentType.objects.get_for_model(instance)
    return typus.name



from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.http import Http404
from django.utils.safestring import mark_safe
from django.utils.text import slugify
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _
from django.views.generic import DetailView
from tinymce.models import HTMLField

from lite_cms_core.managers import BaseEntityManager


CONTENT_STATUS_DRAFT = 1
CONTENT_STATUS_PUBLISHED = 2
CONTENT_STATUS_CHOICES = (
    (CONTENT_STATUS_DRAFT, _("Draft")),
    (CONTENT_STATUS_PUBLISHED, _("Published")),
)


def wrapped_manager(klass):
    if getattr(settings, "USE_MODELTRANSLATION", False):
        from modeltranslation.manager import MultilingualManager
        class Mgr(MultilingualManager, klass):
            pass
        return Mgr()
    return klass()


def _base_concrete_model(abstract, klass):
    for kls in reversed(klass.__mro__):
        if issubclass(kls, abstract) and not kls._meta.abstract:
            return kls
    return None


def base_concrete_model(abstract, model):
    """
    Used in methods of abstract models to find the super-most concrete
    (non abstract) model in the inheritance chain that inherits from the
    given abstract model. This is so the methods in the abstract model can
    query data consistently across the correct concrete model.

    Consider the following::

        class Abstract(models.Model)

            class Meta:
                abstract = True

            def concrete(self):
                return base_concrete_model(Abstract, self)

        class Super(Abstract):
            pass

        class Sub(Super):
            pass

        sub = Sub.objects.create()
        sub.concrete() # returns Super

    """
    if hasattr(model, 'objects'):
        # "model" is a model class
        return (model if model._meta.abstract else
                _base_concrete_model(abstract, model))
    # "model" is a model instance
    return (
        _base_concrete_model(abstract, model.__class__) or
        model.__class__
    )


def unique_slug(queryset, slug_field, slug):
    """
    Ensures a slug is unique for the given queryset, appending
    an integer to its end until the slug is unique.
    """
    i = 0
    while True:
        if i > 0:
            if i > 1:
                slug = slug.rsplit("-", 1)[0]
            slug = f"{slug}-{i}"
        try:
            queryset.get(**{slug_field: slug})
        except ObjectDoesNotExist:
            break
        i += 1
    return slug


class BaseEntity(models.Model):
    """
    Abstract base model for basic cms funcionality.

    """
    class Meta:
        abstract = True

    def __str__(self):
        return self.title

    #: Title of the entity, string with a maxlen of 200.
    title = models.CharField(
        max_length=200, verbose_name=_("Title")
    )
    #: The status of the entity, one of CONTENT_STATUS_DRAFT or CONTENT_STATUS_PUBLISHED
    status = models.IntegerField(
        verbose_name=_("Status"),
        choices=CONTENT_STATUS_CHOICES, default=CONTENT_STATUS_PUBLISHED,
        help_text=_("With Draft chosen, will only be shown for admin users on the site.")
    )
    #: Time and date when the entity is or will be published.
    publish_date = models.DateTimeField(
        _("Published from"),
        help_text=_("With Published chosen, won't be shown until this time"),
        blank=True, null=True, db_index=True
    )
    #: Time and date when the entity is expired, so will not be seen by users anymore.
    expiry_date = models.DateTimeField(
        _("Expires on"),
        help_text=_("With Published chosen, won't be shown after this time"),
        blank=True, null=True
    )

    objects = wrapped_manager(BaseEntityManager)

    @property
    def published(self):
        """
        For non-staff users, return True when status is published and
        the publish and expiry dates fall before and after the
        current date when specified.
        """
        return (self.status == CONTENT_STATUS_PUBLISHED and
                (self.publish_date is None or self.publish_date <= now()) and
                (self.expiry_date is None or self.expiry_date >= now()))

    def _get_next_or_previous_by_publish_date(self, is_next, **kwargs):
        """
        Retrieves next or previous object by publish date. We implement
        our own version instead of Django's so we can hook into the
        published manager and concrete subclasses.
        """
        arg = "publish_date__gt" if is_next else "publish_date__lt"
        order = "publish_date" if is_next else "-publish_date"
        lookup = {arg: self.publish_date}
        concrete_model = base_concrete_model(BaseEntity, self)
        try:
            queryset = concrete_model.objects.published
        except AttributeError:
            queryset = concrete_model.objects.all
        try:
            return queryset(**kwargs).filter(**lookup).order_by(order)[0]
        except IndexError:
            pass
        return None

    def get_next_by_publish_date(self, **kwargs):
        """
        Retrieves next object by publish date.
        """
        return self._get_next_or_previous_by_publish_date(True, **kwargs)

    def get_previous_by_publish_date(self, **kwargs):
        """
        Retrieves previous object by publish date.
        """
        return self._get_next_or_previous_by_publish_date(False, **kwargs)

    def get_absolute_url(self):
        """
        Raise an error if called on a subclass without
        ``get_absolute_url`` defined, to ensure all search results
        contains a URL.
        """
        name = self.__class__.__name__
        msg = f"The model {name} does not have get_absolute_url defined"
        raise NotImplementedError(
            msg
        )

    def save(self, *args, **kwargs):
        if not self.publish_date:
            self.publish_date = now()
        super().save(*args, **kwargs)


class SluggedMixin(models.Model):
    """
    Abstract model that handles auto-generating slugs.

    In default the slug is generated from the title attribute. You can
    specify a different attribute by overriding the ``get_slug()`` method.
    if you want to get the slug from a ``name`` attribute, add this to your class::

            def get_slug(self, attr=None):
                return super().get_slug(attr="name")

    If any instance has the same title as another instance, a
    unique slug is automatically generated by adding a dash and a number.
    """
    slug = models.SlugField(
        max_length=200, blank=True, unique=True,
        help_text=_('Is generated if no value has been specified')
    )

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        """
        If no slug is provided, generates one before saving.
        """
        if not self.slug:
            self.slug = self.generate_unique_slug()
        super().save(*args, **kwargs)

    def generate_unique_slug(self):
        """
        Create a unique slug by passing the result of get_slug() to
        utils.urls.unique_slug, which appends an index if necessary.
        """
        concrete_model = base_concrete_model(SluggedMixin, self)
        slug_qs = concrete_model.objects.exclude(id=self.id)
        return unique_slug(slug_qs, "slug", self.get_slug())

    def get_slug(self, attr=None):
        """
        Allows subclasses to implement their own slug creation logic.
        In default a model 'title' field is required
        """
        if attr is None:
            attr = "title"
        if settings.USE_MODELTRANSLATION:
            # pylint: disable=import-error, import-outside-toplevel
            from modeltranslation.utils import build_localized_fieldname
            attr = build_localized_fieldname(attr, settings.LANGUAGE_CODE)
        return slugify(getattr(self, attr, None) or self.title)

    def admin_link(self):
        """
        Provides a "view on site" link for the admin site.
        """
        return mark_safe(
            f"<a href='{self.get_absolute_url()}'>{_('View on site')}</a>"
        )
    admin_link.short_description = ""


class TimeStampedMixin(models.Model):
    """
    Provides timestamps for ``created`` and ``modified`` fields. Both fields
    are read-only.
    """

    class Meta:
        abstract = True

    #: Time and date when the instance was created.
    created = models.DateTimeField(auto_now_add=True, verbose_name=_('Created'))
    #: Time and date of the latest change of the instance.
    changed = models.DateTimeField(auto_now=True, verbose_name=_('Changed'))


class ContentFieldMixin(models.Model):
    """
    Get a HTML edit field in model
    """

    class Meta:
        abstract = True

    content = HTMLField(verbose_name=_("Content"), null=True, blank=True)


class AdminOrderMixin(models.Model):
    """
    Let the user order the model instances in admin
    """

    class Meta:
        abstract = True

    sort_order = models.PositiveIntegerField(default=0)


class PreventUnpublishedAccessDetailView(DetailView):
    """
    In default an admin user will be shown model instances in both status,
    ``draft`` and ``published``. If you want to hide the *draft* items from
    non-admin users, you can use this subclass of the Django DetailView.

    This is only neccessary for DetalView since in ListView you would
    select the instances by ``Model.objects.published()``.
    """

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_staff:
            if not self.get_object().published:
                raise Http404
        return super().dispatch(request, *args, **kwargs)



from django.contrib.auth.views import LogoutView
from django.urls import path

from lite_cms_core import views

app_name = 'lite_cms_core'

urlpatterns = [
    path('search/', views.search, name='search'),
    path('extsearch/', views.ext_search_form, name='extsearch'),
    path('logout/', LogoutView.as_view(), name='logout'),
]



from operator import itemgetter

from django.apps import apps
from django.conf import settings
from django.core.paginator import Paginator
from django.shortcuts import render
from django.utils.translation import gettext_lazy as _


def search(request, template="lite_cms_core/search_results.html", extra_context=None):
    """
    Display search results. Takes an optional "contenttype" GET parameter
    in the form "app-name.ModelName" to limit search results to a single model.
    """
    query = request.GET.get("q", "")
    try:
        parts = request.GET.get("type", "").split(".", 1)
        search_model = apps.get_model(*parts)
    except (ValueError, TypeError, LookupError, AttributeError):
        search_types = settings.SEARCH_MODEL_CHOICES
        results = []
        for search_type in search_types:
            parts = search_type.split(".", 1)
            search_model = apps.get_model(*parts)
            results += search_model.objects.search(query, for_user=request.user)
        search_type = _("Everything")
    else:
        search_type = search_model._meta.verbose_name_plural.capitalize()
        results = search_model.objects.search(query, for_user=request.user)

    paginator = Paginator(results, 10)
    page_obj = paginator.get_page(request.GET.get('page'))

    context = {"query": query, "results": page_obj,
               "search_type": search_type, 'total': len(results)}
    context.update(extra_context or {})
    return render(request, template, context)


def ext_search_form(request):
    choices = set()
    for name in settings.SEARCH_MODEL_CHOICES:
        try:
            model = apps.get_model(*name.split(".", 1))
            model_name = model._meta.verbose_name.title()
        except LookupError:
            pass
        else:
            choices.add((model_name, name))
    search_choices = list(choices)
    # sort by localized name
    search_choices.sort(key=itemgetter(0))
    return render(request, 'lite_cms_core/ext_search_form.html', {
        'search_model_choices': search_choices
    })



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='django-lite-cms-core',
    version='0.0.4',
    packages=setuptools.find_packages(
        exclude=["lite_cms_core_sample", "*manage.py"]
    ),
    include_package_data=True,
    package_data={
        "lite_cms_core": [
            "templates/lite_cms_core/*.html",
            "templates/lite_cms_core/includes/*.html",
            "locale/*/LC_MESSAGES/*",
        ],
    },
    description='CMS core classes for Django.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Christian Wiegand',
    license='MIT',
    url='https://github.com/christianwgd/django-lite-cms-core',
    keywords=['django', 'bootstrap', 'cms'],
    install_requires=[
        'django',
        'django-admin-sortable2',
        'django-bootstrap5',
        'django-filebrowser-no-grappelli',
        'django-tinymce',
        'Pillow',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.10',
)
